@@ -1,5 +1,5 @@
 package Net::Hadoop::YARN::ResourceManager;
-$Net::Hadoop::YARN::ResourceManager::VERSION = '0.004';
+$Net::Hadoop::YARN::ResourceManager::VERSION = '0.100';
 {
   $Net::Hadoop::YARN::ResourceManager::DIST = 'Net-Hadoop-YARN';
 }
@@ -7,32 +7,92 @@ $Net::Hadoop::YARN::ResourceManager::VERSION = '0.004';
 use strict;
 use warnings;
 use 5.10.0;
-use Moo;
+
 use Data::Dumper;
+use Moo;
+use Ref::Util qw(
+    is_ref
+    is_arrayref
+    is_hashref
+);
+use Scalar::Util qw(
+    refaddr
+);
 
 with 'Net::Hadoop::YARN::Roles::Common';
 
 has '+servers' => ( default => sub { ["localhost:8088"] }, );
 
+has '+add_host_key' => ( default => sub { 1 } );
+
+
+sub active_rm {
+    my $self = shift;
+    my $opt  = is_hashref $_[0] ? shift @_ : {};
+    my $rv;
+
+    foreach my $server ( @{ $self->servers } ) {
+        my $info = $self->info({ server => $server });
+        my $haState = $info->{haState} || next;
+        if ( $haState eq 'ACTIVE' ) {
+            $rv = $server;
+            last;
+        }
+    }
+
+    if ( ! $rv ) {
+        die sprintf "Failed to locate the active YARN Resource Manager from these hosts: %s",
+                        join( q{, }, @{ $self->servers } ),
+        ;
+    }
+
+    if ( $opt->{hostname_only} ) {
+        return +( split m{[:]}xms, $rv )[0];
+    }
+
+    return $rv;
+}
 
 sub info {
     my $self = shift;
-    my $res = $self->_get("cluster/info");
-    return $res->{clusterInfo} || $res;
+    my $opt  = is_hashref $_[0] ? shift @_ : {};
+
+    my $res = $self->_get(
+                    "cluster/info",
+                    undef,
+                    ( $opt->{server} or () ),
+                );
+
+    return $self->_apply_host_key(
+                $res,
+                $res->{clusterInfo} || $res,
+            );
 }
 
 
 sub metrics {
     my $self = shift;
-    my $res = $self->_get("cluster/metrics");
-    return $res->{clusterMetrics} || $res;
+    my $opt  = is_hashref $_[0] ? shift @_ : {};
+    my $res = $self->_get(
+                    "cluster/metrics",
+                    undef,
+                    ( $opt->{server} or () ),
+                );
+
+    return $self->_apply_host_key(
+                $res,
+                $res->{clusterMetrics} || $res,
+            );
 }
 
 
 sub scheduler {
     my $self = shift;
-    my $res = $self->_get("cluster/scheduler");
-    return $res->{schedulerInfo} || $res;
+    my $res  = $self->_get("cluster/scheduler");
+    return $self->_apply_host_key(
+                $res,
+                $res->{schedulerInfo} || $res,
+            );
 }
 
 
@@ -54,7 +114,11 @@ sub apps {
     my $res = $self->_get(
         $app_id ? "cluster/apps/$app_id" : ( "cluster/apps", { params => $options } )
     );
-    return $res->{apps}{app} || $res->{app} || $res;
+
+    return $self->_apply_host_key(
+                $res,
+                $res->{apps}{app} || $res->{app} || $res,
+            );
 }
 
 
@@ -77,11 +141,20 @@ sub appstatistics {
     elsif ( @_ > 1 ) {
         $options = {@_};
     }
-    my $res
-        = $self->_get( "cluster/appstatistics", $options ? { params => $options } : () );
+    my $res = $self->_get(
+                    "cluster/appstatistics",
+                    ( $options ? {
+                        params => $options,
+                    } : () ),
+                );
+
     if ($res) {
-        return $res->{appStatInfo}{statItem} || $res->{statItem};
+        return $self->_apply_host_key(
+                    $res,
+                    $res->{appStatInfo}{statItem} || $res->{statItem},
+                );
     }
+
     return;
 }
 
@@ -102,11 +175,53 @@ sub nodes {
         $options = {@_};
     }
     my $res = $self->_get(
-        $node_id ? "cluster/nodes/$node_id" : ( "cluster/nodes", { params => $options } ) );
-    return $res->{nodes}{node} || $res->{node} || $res;
+                    $node_id ? "cluster/nodes/$node_id"
+                             :  (
+                                    "cluster/nodes",
+                                    { params => $options },
+                                )
+                );
+
+
+    return $self->_apply_host_key(
+                $res,
+                $res->{nodes}{node} || $res->{node} || $res,
+            );
+
 }
 
 
+sub _apply_host_key {
+    my $self = shift;
+    my $res  = shift;
+    my $rv   = shift;
+
+
+    if (   is_ref( $res )
+        && is_ref( $rv )
+        && ( refaddr $res eq refaddr $rv )
+    ) {
+        return $rv;
+    }
+
+    my $host_key  = $self->host_key;
+    my $this_host = $res->{ $host_key };
+
+    if ( is_hashref $rv ) {
+        $rv->{ $host_key } = $this_host;
+    }
+    elsif ( is_arrayref $rv ) {
+        foreach my $e ( @{ $rv } ) {
+            $e->{ $host_key } = $this_host;
+        }
+    }
+    else {
+        die "Got unknown data: $rv";
+    }
+
+    return $rv;
+}
+
 1;
 
 __END__
@@ -121,7 +236,7 @@ Net::Hadoop::YARN::ResourceManager
 
 =head1 VERSION
 
-version 0.004
+version 0.100
 
 =head1 METHODS
 
