@@ -1,5 +1,5 @@
 package Net::Hadoop::YARN::Roles::Common;
-$Net::Hadoop::YARN::Roles::Common::VERSION = '0.004';
+$Net::Hadoop::YARN::Roles::Common::VERSION = '0.100';
 {
   $Net::Hadoop::YARN::Roles::Common::DIST = 'Net-Hadoop-YARN';
 }
@@ -20,7 +20,6 @@ use Socket;
 use URI;
 use XML::LibXML::Simple;
 
-
 has _json => (
     is      => 'rw',
     lazy    => 1,
@@ -81,6 +80,18 @@ has servers => (
     lazy => 1,
 );
 
+has add_host_key => (
+    is      => 'rw',
+    default => sub { 0 },
+    lazy    => 1,
+);
+
+has host_key => (
+    is  => 'rw',
+    default => sub { '__RESTHost' },
+    lazy => 1,
+);
+
 sub _check_host {
     my $host = shift;
     return !!( eval { inet_aton($host) }
@@ -94,9 +105,9 @@ sub _check_servers {
         if (   ! _check_host($host)
             || $port !~ /^[0-9]+$/
             || $port < 1
-            || $port > 65535
+            || $port > 19888
         ) {
-            die "server $server bad host";
+            die "server $server bad host (port=$port)";
         }
     }
     return 1;
@@ -130,7 +141,10 @@ sub _post {
 
 sub _request {
     my $self     = shift;
-    my @servers  = @{ $self->servers };
+    my ( $method, $path, $extra, $server ) = @_;
+
+    my $host_key = $self->host_key;
+    my @servers  = $server ? ( $server ) : @{ $self->servers };
     my $maxtries = @servers;
 
     my ($eval_error, $ret, $n);
@@ -138,6 +152,7 @@ sub _request {
     # get a copy, don't mess with the global setting
     #
     my @banned_servers;
+    my $selected_server;
 
     TRY: for ( 1 .. $maxtries ) {
         my $redo;
@@ -153,12 +168,12 @@ sub _request {
             last TRY;
         }
 
+        $selected_server = $servers[0];
         eval {
             $eval_error = undef;
-            my ( $method, $path, $extra ) = @_;
 
             my $uri = $self->_mk_uri(
-                            $servers[0],
+                            $selected_server,
                             $path,
                             $method eq 'GET' ? $extra->{params} : (),
                         );
@@ -191,7 +206,7 @@ sub _request {
                     }xms ) {
                         push @banned_servers, shift @servers;
                         $redo++;
-                        die "Hit the standby with $servers[0]";
+                        die "Hit the standby with $selected_server";
                     }
                     die "Response doesn't look like XML: $content";
                 }
@@ -252,7 +267,14 @@ sub _request {
             push @servers, shift @servers if @servers > 1;
         };
 
-        last TRY if $ret;
+        if ( $ret ) {
+            if ( $self->add_host_key ) {
+                # mark where we've been
+                $ret->{ $host_key } = $selected_server;
+            }
+            last TRY;
+        }
+
     } # retry as many times as there are servers
 
     if ( $eval_error ) {
@@ -276,7 +298,7 @@ Net::Hadoop::YARN::Roles::Common
 
 =head1 VERSION
 
-version 0.004
+version 0.100
 
 =head1 AUTHOR
 
